﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=8
@EndOfDesignText@
'Static code module
Sub Process_Globals
	
	Type HTMLPage(PageName As String, Page As String)
	Public const OUTPUTMODE_BBCODE As String = "BBCode"
	Public const OUTPUTMODE_BBCODEEXPORT As String = "BBCodeExport"
	Public const OUTPUTMODE_HTML As String = "HTML"
	Public const OUTPUTMODE_TEXT As String = "Text"
	
	Private mCSSFileName As String = "builddoc-style.css"
	Private mPages As List
	Private mHeader As String
	Private mStyle As String
	Private H1, H1OFF As String
	Private H2, H2OFF As String														'ignore
	Private H3, H3OFF As String
	Private Small, SmallOff, Big, BigOff, Bigger, BiggerOff  As String				'ignore
	Private TabSpace As String
	Private Ital, ItalOff, Bold, BoldOff As String
	Private Indent1, Indent2, IndentOff, Indent2Off As String
	Private CommentColor, PTypeColor, KeywordColor, ColorOff As String
	Private Plain, PlainOff As String
	Private LineBreak,Linebreak2 As String
	Public IsInitialized As Boolean
	Public mShowComments As Boolean = True
	
End Sub

'Create the pages for display contained in the current Library
Public Sub CreatePages(LibraryName As String,Style As String,ShowComments As Boolean) As List
	
	Dim Pages As List
	Pages.Initialize
	
	If ProcessXML.Library.IsInitialized = False Then
		Return Pages
	End If
	
	mShowComments = ShowComments
	mStyle = Style
	
	If mStyle = OUTPUTMODE_BBCODE Then
		FormatBBC
	Else If mStyle = OUTPUTMODE_BBCODEEXPORT Then
		FormatBBCForum
	Else If mStyle = OUTPUTMODE_HTML Then
		FormatHTML
	Else If mStyle = OUTPUTMODE_TEXT Then
		ClearFormat
	End If
	
	mHeader = HeaderRow($"Version ${ProcessXML.Library.Version}"$,$"${LibraryName}"$,$"Author ${ProcessXML.Library.Author}"$)
	
	For i = 0 To ProcessXML.Library.Classes.Size - 1
		Dim Class As ClassType = ProcessXML.Library.Classes.Get(i)
		Dim Text As String = ParseClass(Class)
		Pages.Add(Text)
	Next
	
	If Pages.Size > 1 And mStyle <> OUTPUTMODE_BBCODEEXPORT Then
		If mStyle <> OUTPUTMODE_BBCODE Then
			For i = 0 To Pages.Size - 1
				Dim Text As String = GetBookmarks(Pages,i) & LineBreak & Pages.Get(i)
				Pages.Set(i,Text)
			Next
		End If
		CreateIndex(Pages)
	End If
	
	IsInitialized = True
	
	Select mStyle
		Case OUTPUTMODE_BBCODE
			mPages = Pages
		
		Case OUTPUTMODE_HTML
			If Pages.Size > 1 Then
				For i = 0 To Pages.Size - 1
					Dim HTML As HTMLPage
					HTML.Initialize
					If i = 0 Then
						HTML.PageName = "Index.html"
						HTML.Page = HTMLHeader & mHeader & LineBreak & Pages.Get(i) & HTMLFooter
						Pages.Set(i,HTML)
					Else
						Dim Class As ClassType = ProcessXML.Library.Classes.Get(i - 1)
						If Class.ShortName <> "" Then
							HTML.PageName = Class.ShortName & ".html"
						Else
							HTML.PageName = Class.Name & ".html"
						End If
						HTML.Page = HTMLHeader &  mHeader & LineBreak & Pages.Get(i) & HTMLFooter
						Pages.Set(i,HTML)
					End If
				Next
			Else
				Dim HTML As HTMLPage
				HTML.Initialize
				Dim Class As ClassType = ProcessXML.Library.Classes.Get(0)
				If Class.ShortName <> "" Then
					HTML.PageName = Class.ShortName & ".html"
				Else
					HTML.PageName = Class.Name & ".html"
				End If
				HTML.Page = HTMLHeader & Pages.Get(0) & HTMLFooter
				Pages.Set(0,HTML)
			End If
			'Reset style as it has been set to generate other output
			mStyle = OUTPUTMODE_BBCODE
		Case Else
			'Reset style as it has been set to generate other output
			mStyle = OUTPUTMODE_BBCODE
	End Select
	
	Return Pages
	
End Sub

'Generates the CSS string for the external stylesheet for HTML export
Public Sub CSS As String
	Return $"body{
	background-color: lightblue;
	font-family:Arial; 
	}
	div.wrap{
		width:960px;
		margin:0 auto;
		background-color: white;
		padding:20px;
		min-height: 100%;
	}
	.index-title{
	width:100%;
	text-align:center;
	}
"$
End Sub

'Get the CSS filename that the pages are expecting
Public Sub CSSFileName As String
	Return mCSSFileName
End Sub

'Get bookmarks used once the pages are created
Public Sub GetWorkingBookmarks(PageNo As Int) As String
	If mStyle = OUTPUTMODE_BBCODE Then
		Dim LeftStr As String = " "
		Dim RightStr As String = " "
		
		'There is no index page In Library.Classes
		PageNo = PageNo - 1
		
		If PageNo > 0 Then
			Dim Class As ClassType = ProcessXML.Library.Classes.Get(PageNo - 1)
			Dim ClassName As String
			If Class.ShortName = "" Then
				ClassName = Class.Name
			Else
				ClassName = Class.ShortName
			End If
			LeftStr = $"[url="${PageNo}"][FontAwesome=0xF053/] ${ClassName}[/url]"$
		End If
	
		If PageNo < mPages.Size - 2 Then
			Dim Class As ClassType = ProcessXML.Library.Classes.Get(PageNo + 1)
			Dim ClassName As String
			If Class.ShortName = "" Then
				ClassName = Class.Name
			Else
				ClassName = Class.ShortName
			End If
			RightStr = $"[url="${PageNo + 2}"]${ClassName} [FontAwesome=0xF054/][/url]"$
		End If
		If ProcessXML.Library.Classes.Size > 1 Then Return IndexRow($"[url="${0}"]Index[/url]"$,LeftStr,RightStr)
	End If
	Return ""
End Sub

'Get the bookmarks string, used when creating the pages
Private Sub GetBookmarks(Pages As List,PageNo As Int) As String
	Dim LeftStr As String = " "
	Dim RightStr As String = " "
	
	If PageNo < 0 Or PageNo > Pages.Size - 1 Then Return ""
	
	Select mStyle
	
		Case OUTPUTMODE_BBCODE
			If PageNo > 0 Then
				Dim Class As ClassType = ProcessXML.Library.Classes.Get(PageNo - 1)
				Dim ClassName As String
				If Class.ShortName = "" Then
					ClassName = Class.Name
				Else
					ClassName = Class.ShortName
				End If
				LeftStr = $"[url="${PageNo}"][FontAwesome=0xF053/] ${ClassName}[/url]"$
			End If
	
			If PageNo < Pages.Size - 1 Then
				Dim Class As ClassType = ProcessXML.Library.Classes.Get(PageNo + 1)
				Dim ClassName As String
				If Class.ShortName = "" Then
					ClassName = Class.Name
				Else
					ClassName = Class.ShortName
				End If
				RightStr = $"[url="${PageNo + 2}"]${ClassName} [FontAwesome=0xF054/][/url]"$
			End If
	
			Return IndexRow($"[url="${0}"]Index[/url]"$,LeftStr,RightStr)
		Case OUTPUTMODE_HTML
			If PageNo > 0 Then
				Dim Class As ClassType = ProcessXML.Library.Classes.Get(PageNo - 1)
				Dim LinkName As String
				If Class.ShortName <> "" Then
					LinkName = Class.ShortName & ".html"
				Else
					LinkName = Class.Name & ".html"
				End If
				LeftStr = $"<a href="${LinkName}">&lt; ${Class.Name}</a>"$
			End If
	
			If PageNo < Pages.Size - 1 Then
				Dim Class As ClassType = ProcessXML.Library.Classes.Get(PageNo + 1)
				Dim LinkName As String
				If Class.ShortName <> "" Then
					LinkName = Class.ShortName & ".html"
				Else
					LinkName = Class.Name & ".html"
				End If
				RightStr = $"<a href="${LinkName}">${Class.Name} &gt;</a>"$
			End If
	
			Return IndexRow($"<a href="Index.html">Index</a>"$,LeftStr,RightStr)
		Return ""
		Case Else
		Return ""
	End Select
End Sub

'Create the index page
Private Sub CreateIndex(Pages As List)
	Dim Text As String
	Select mStyle
		Case OUTPUTMODE_BBCODE
'			Text = Text & $"[url="${0}"]Index[/url] ${LineBreak}"$
			For i = 0 To ProcessXML.Library.Classes.Size - 1
				Dim Class As ClassType = ProcessXML.Library.Classes.Get(i)
				Dim ClassName As String
				If Class.ShortName <> "" Then
					ClassName = Class.ShortName
				Else
					ClassName = Class.Name
				End If
				Text = Text & $"[url="${i+1}"]${ClassName}[/url] ${LineBreak}"$
			Next
			Pages.InsertAt(0,Text)
		Case OUTPUTMODE_HTML
			Text = Text & $"<div class="index-title"><p>Library documentation created by BuildDoc for B4x</p></div>"$
			Text = Text & $"<p class="index-subtitle">Contents</p>"$
			Text = Text & $"<a href="Index.html">Index</a>${LineBreak}"$
			For i = 0 To ProcessXML.Library.Classes.Size - 1
				Dim Class As ClassType = ProcessXML.Library.Classes.Get(i)
				Dim ClassName As String
				If Class.ShortName <> "" Then
					ClassName = Class.ShortName
				Else
					ClassName = Class.Name
				End If
				Text = Text & $"<a href="${ClassName & ".html"}">${ClassName}</a>${LineBreak}"$
			Next
			Pages.InsertAt(0,Text)
		Case Else
		'No Index
	End Select
	
End Sub

'Create a header row
Private Sub HeaderRow(Field1 As String, Field2 As String, Field3 As String) As String
	If mStyle = OUTPUTMODE_BBCODE Then Return $"[Span MinWidth=20%x Alignment=left]${Small}${Field1}${SmallOff}[/Span][Span MinWidth=40%x Alignment=center]${H1}${Field2}${H1OFF}[/Span][Span MinWidth=20%x Alignment=right]${Small}${Field3}${SmallOff}[/Span]"$
	If mStyle = OUTPUTMODE_HTML Then Return  $"<table style="width:100%">
  <tr>
    <td width="20%" align="center">${Small}${Field1}${SmallOff}</td>
    <td width="60%" align="center">${H1}${Field2}${H1OFF}</td>
	<td width="20%" align="center">${Small}${Field3}${SmallOff}</td>
  </tr>
</table>"$
	If mStyle = OUTPUTMODE_TEXT Then Return $"${Field1}${TAB}${Field2}${TAB}${Field3}"$
	Return ""
End Sub

Private Sub IndexRow(Field1 As String, Field2 As String,Field3 As String) As String
	If mStyle = OUTPUTMODE_BBCODE Then Return $"[Span MinWidth=20%x Alignment=left]${Field1}[/Span][Span MinWidth=28%x Alignment=right]${Field2}[/Span] - [Span MinWidth=28%x Alignment=left]${Field3}[/Span][Span MinWidth=20%x Alignment=right] [/Span]"$
	If mStyle = OUTPUTMODE_HTML Then Return $"<table style="width:100%">
  <tr>
    <td width="20%" align="center">${Field1}</td>
    <td width="60%" align="center">${Field2} - ${Field3}</td>
	<td width="20%" align="center"> </td>
  </tr>
</table>"$
	If mStyle = OUTPUTMODE_TEXT Then Return ""
	Return ""
End Sub

Private Sub ParseClass(Class As ClassType) As String
	
	Dim Text As String = $"${LineBreak}${Bold}${Bigger}Class : ${BiggerOff}${BoldOff}${Class.Name}${TabSpace}${Bold}${Bigger}ShortName : ${BiggerOff}${BoldOff} ${Class.ShortName}${LineBreak}"$
	
	If Class.Events.Size > 0 Then
		Text = Text & $"${LineBreak}${Indent1}${H3}${Bold}Events:${BoldOff}${H3OFF}${IndentOff}${Linebreak2}"$
		For Each Item As ItemType In Class.Events
			Text = Text & ParseItem($"${Indent2}"$,Item) & Linebreak2
		Next
	End If
	
	If Class.Fields.Size > 0 Then
		Text = Text & $"${LineBreak}${Indent1}${H3}${Bold}Fields:${BoldOff}${H3OFF}${IndentOff}${Linebreak2}"$
		For Each Item As ItemType In Class.Fields
			Text = Text & ParseItem($"${Indent2}"$,Item) & Linebreak2
		Next
	End If
	
	If Class.Properties.Size > 0 Then
		Text = Text & $"${LineBreak}${Indent1}${H3}${Bold}Properties:${BoldOff}${H3OFF}${IndentOff}${Linebreak2}"$
		For Each Item As ItemType In Class.Properties
			Text = Text & ParseItem($"${Indent2}"$,Item) & Linebreak2
		Next
	End If
	
	If Class.Methods.Size > 0 Then
		Text = Text & $"${LineBreak}${Indent1}${H3}${Bold}Methods:${BoldOff}${H3OFF}${IndentOff}${Linebreak2}"$
		For Each Item As ItemType In Class.Methods
			Text = Text & $"${ParseItem(Indent2,Item)} ${Linebreak2}"$
		Next
	End If
	
	Return Text
End Sub

Private Sub ParseItem(Indent As String,Item As ItemType) As String
	Dim IndOff As String
	If Indent =Indent1 Then IndOff = IndentOff
	If Indent =Indent2 Then IndOff = Indent2Off
	Dim Text As String = $"${Indent}${Bold}${Item.Name}${BoldOff}"$
	If Item.Parameters.Size > 0 Then
		Text = Text & "("
		Dim Params As String
		For Each Param As ParamaterType In Item.Parameters
			Params = Params & $", ${Param.Name} ${KeywordColor}As${ColorOff} ${PTypeColor}${Param.PType}${ColorOff}"$
		Next
		Text = Text & Params.SubString(2)
		Text = Text & ") "
	End If
	
	If Item.ReturnType <> "" Then
		Text = Text & $" ${KeywordColor}As${ColorOff} ${PTypeColor}${Item.ReturnType.Replace("[]","()")}${ColorOff}"$
	End If

	Select Item.READWRITE
		Case ProcessXML.READWRITE_BOTH
		Case ProcessXML.READWRITE_READONLY
			Text = Text & $"${Ital}[Read Only]${ItalOff}"$
		Case ProcessXML.READWRITE_WRITEONLY
			Text = Text & $"${Ital}[Write Only]${ItalOff}"$
	End Select
	
	If mShowComments And Item.Comment <> "" Then Text = Text & $"${LineBreak}${CommentColor}${ParseComments(Item.Comment)}${ColorOff}"$
	
	Text = Text & IndOff
	Return Text
	End Sub

Private Sub ParseComments(Comments As String) As String
	Comments = Comments.Replace("&lt;code&gt;","<code>").Replace("&amp;","&") _
	.Replace("&lt;I&gt;",Ital).Replace("<I>",Ital) _
	.Replace("&lt;/I&gt;",ItalOff).Replace("</I>",ItalOff)
	Return Plain & Comments.Trim & PlainOff
End Sub

'Get a generated page text
Public Sub GetPage(Index As Int) As String
	If Index < 0 Or Index > mPages.Size Then Return ""
	Return mPages.Get(Index)
End Sub

'Get all generated pages text as a list
Public Sub GetPages As List
	Return mPages
End Sub

'Get the page header
Public Sub GetHeader As String
	Return mHeader
End Sub

'Get the page count
Public Sub PageCount As Int
	Return mPages.Size - 1
End Sub

'Get the Top part of the HTML page
Private Sub HTMLHeader As String
	Return $"<html>
	<head>
	<link rel="stylesheet" type="text/css" href="${mCSSFileName}">
	</head>
	<body>
		<div class="wrap">
"$
End Sub

'Get the bottom part of the HTML page
Private Sub HTMLFooter As String
	Return $"</div>
	</body>
</html>"$
End Sub

Private Sub FormatBBC
	Bold = "[B]"
	BoldOff = "[/B]"
	H1 = "[TextSize=25]"
	H1OFF = $"[/Textsize]"$
	H2 = "[TextSize=18]"
	H2OFF = H1OFF
	H3 = "[TextSize=16]"
	H3OFF = H1OFF
	Big = "[TextSize=18]"
	BigOff = H1OFF
	Bigger = "[TextSize=18]"
	BiggerOff = H1OFF
	TabSpace = TAB
	Small = "[TextSize=11]"
	SmallOff = H1OFF
	Ital = "[font=italic]"
	ItalOff = "[/font]"
	Indent1 = "[Indent=1]"
	Indent2 = "[Indent=2]"
	IndentOff = "[Indent=0]"
	Indent2Off = "[Indent=0]"
	CommentColor = "[color=0x00008000]"
	KeywordColor = "[color=0x000000FF]"
	PTypeColor = "[color=0x00008b8b]"
	ColorOff = "[/color]"
	Plain = "[Plain]"
	PlainOff = "[/Plain]"
	LineBreak = CRLF
	Linebreak2 = CRLF & CRLF
End Sub
Private Sub FormatBBCForum
	Bold = "[b]"
	BoldOff = "[/b]"
	H1 = "[size=25]"
	H1OFF = $"[/size]"$
	H2 = "[size=18]"
	H2OFF = H1OFF
	H3 = "[size=16]"
	H3OFF = H1OFF
	Big = "[size=18]"
	BigOff = H1OFF
	Bigger = "[size=18]"
	BiggerOff = H1OFF
	TabSpace = TAB
	Small = "[size=11]"
	SmallOff = H1OFF
	Ital = "[i]"
	ItalOff = "[/i]"
	Indent1 = "[Indent]"
	Indent2 = "[Indent][Indent]"
	IndentOff = "[/Indent]"
	Indent2Off = "[/Indent][/Indent]"
	CommentColor = "[color=#008000]"
	KeywordColor = "[color=#0000FF]"
	PTypeColor = "[color=#008b8b]"
	ColorOff = "[/color]"
	Plain = ""
	PlainOff = ""
	LineBreak = CRLF
	Linebreak2 = CRLF & CRLF
End Sub
Private Sub FormatHTML
	Bold = "<b>"
	BoldOff = "</b>"
	H1 = "<h1>"
	H1OFF = $"</h1>"$
	H2 = "<h2>"
	H2OFF = "</h2>"
	H3 = "<h3>"
	H3OFF = "</h3>"
	Big = $"<font size="+1">"$
	BigOff = "</font>"
	Bigger = $"<font size="+2">"$
	BiggerOff = "</font>"
	Small = $"<font size="-1">"$
	SmallOff = "</font>"
	TabSpace = "&nbsp;&nbsp;&nbsp;&nbsp;"
	Ital = "<i>"
	ItalOff = "</i>"
	Indent1 = $"<p style="margin-left: 20px">"$
	Indent2 = $"<p style="margin-left: 40px">"$
	IndentOff = "</p>"
	Indent2Off = "</p>"
	CommentColor = $"<font color="#008000">"$
	KeywordColor = $"<font color="#0000FF">"$
	PTypeColor = $"<font color="#008b8b">"$
	ColorOff = "</font>"
	Plain = ""'"<pre>"
	PlainOff = ""'"</pre>"
	LineBreak = "<br />"
	Linebreak2 = CRLF
	
End Sub

Private Sub ClearFormat
	Bold = ""
	BoldOff = ""
	H1 = ""
	H1OFF = ""
	H2 = ""
	H2OFF = ""
	H3 = ""
	H3OFF = ""
	Small = ""
	SmallOff = ""
	Big = ""
	BigOff = ""
	Bigger = ""
	BiggerOff = ""
	TabSpace = TAB
	Ital = ""
	ItalOff = ""
	Indent1 = ""
	Indent2 = ""
	IndentOff = ""
	Indent2Off = ""
	CommentColor = ""
	KeywordColor = ""
	PTypeColor = ""
	ColorOff = ""
	Plain = ""
	PlainOff = ""
	LineBreak = CRLF
	Linebreak2 = CRLF & CRLF
End Sub