﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=8
@EndOfDesignText@
'Static code module
Sub Process_Globals
	Public Const READWRITE_BOTH As Int = 0
	Public Const READWRITE_READONLY As Int = 1
	Public Const READWRITE_WRITEONLY As Int = 2
	
	Public CONST PARSE_XMLNOTVALID As Int = -1
	Public CONST PARSE_XMLNOTB4XLIB As Int = -2
	Public CONST PARSE_OK As Int = 1
	
	Private mXML As String
	
	Type LibraryType(Classes As List, Author As String, Version As String)
	Type ClassType(Name As String, ShortName As String, Properties As List, Fields As List, Methods As List,Events As List)
	Type ItemType(ItemType As String, Name As String, Comment As String,ReturnType As String, Parameters As List, READWRITE As Int)
	Type ParamaterType(Name As String, PType As String)
	
	Private XMLMap As Map
	Private mLibrary As LibraryType
End Sub

'Clear all data
Public Sub ClearData
	mXML = ""
	XMLMap.Initialize
	Dim mLibrary As LibraryType
End Sub

'Process an XML file
Public Sub Process2(FileName As String) As Int
	Dim XML As String = File.ReadString(FileName,"")
	Return Process(XML)
End Sub

'Process an XML string
Public Sub Process(XML As String) As Int
	mXML = XML
	XMLMap.Initialize
	Dim XML2M As Xml2Map
	XML2M.Initialize
	Try
		XMLMap = XML2M.Parse(XML)
	Catch
		Return PARSE_XMLNOTVALID
	End Try
	Return ProcessDo
End Sub

Private Sub ProcessDo As Int
	mLibrary.Initialize
	mLibrary.Classes.Initialize
	
	Dim Root As Map = GetSingleElement(XMLMap,"root")
	
	Dim Test As List = GetElementsFromMap(Root,"doclet-version-NOT-library-version")
	If Test.Size = 0 Then
		Return PARSE_XMLNOTB4XLIB
	End If
	
	Dim Classes As List = GetElementsFromMap(Root,"class")
	If Classes.IsInitialized Then
		For Each M As Map In Classes
			Dim Class As ClassType
			Class.Initialize
			
			Class.Name = GetSingleElement(M,"name")
			Class.ShortName = GetSingleElement(M,"shortname")
			Class.Events= ParseEvents(GetElementsFromMap(M,"event"))
			
			Class.Properties = ParseItems(GetElementsFromMap(M,"property"),"property")
			Class.Fields = ParseItems(GetElementsFromMap(M,"field"),"field")
			Class.Methods = ParseItems(GetElementsFromMap(M,"method"),"method")
			
			mLibrary.Classes.Add(Class)
		Next
	End If
	
	mLibrary.Version = GetSingleElement(Root,"version")
	mLibrary.Author = GetSingleElement(Root,"author")
	
	Return PARSE_OK
End Sub

Private Sub ParseEvents(Events As List) As List
	Dim Results As List
	Results.Initialize
	For Each Line As String In Events
		Dim m As Matcher = Regex.Matcher2("^\s*([^\s(]+)", Regex.CASE_INSENSITIVE, Line)
		If m.Find = False Then Continue
		Dim Item As ItemType
		Item.Initialize
		Item.ItemType = "event"
		Item.Name = m.Group(1)
		Item.Parameters.Initialize
		Dim i1 As Int = Line.IndexOf("(")
		Dim i2 As Int = Line.LastIndexOf(")")
		If i1 > -1 And i2 > -1 Then
			m = Regex.Matcher2("([\w_]+)(?:\(\))?\s+as\s+([\w_]+)", Regex.CASE_INSENSITIVE, Line.SubString2(i1 + 1, i2))
			Do While m.Find
				Dim p As ParamaterType
				p.Initialize
				p.Name = m.Group(1)
				p.PType = m.Group(2)
				Item.Parameters.Add(p)
			Loop
		End If
		Results.Add(Item)
	Next
	Return Results
End Sub

Private Sub ParseItems(Items As List,ItemType As String) As List

	Dim ReturnItems As List
	ReturnItems.Initialize
	If Items.IsInitialized Then
		For Each Item As Map In Items
			Dim IT As ItemType
			IT.Initialize
			IT.Parameters.Initialize
			IT.ItemType = ItemType
			IT.Name = GetSingleElement(Item,"name")
			IT.Comment = GetSingleElement(Item,"comment")
			IT.ReturnType = GetSingleElement(Item,"returntype")
			Dim Parameters As List = GetElementsFromMap(Item,"parameter")
			If Parameters.IsInitialized Then
				For Each Param As Map In Parameters
					Dim P As ParamaterType
					P.Initialize
					P.Name = GetSingleElement(Param,"name")
					P.PType = GetSingleElement(Param,"type")
					IT.Parameters.Add(P)
				Next
			End If
			
			ReturnItems.Add(IT)
		Next
	End If
	Return ReturnItems
End Sub

Private Sub GetSingleElement(M As Map, Key As String) As Object
	Dim L As List = GetElementsFromMap(M,Key)
	If L.IsInitialized And L.Size > 0 Then
		Dim O As Object = L.Get(0)
		If O Is Map And Key = "name" Then
			Dim M As Map = O
			Dim O As Object = M.Get("Attributes")
			If O Is Map Then
				Dim m As Map = o
				Return M.GetDefault("DesignerName",M.GetDefault("name",""))
			End If
		End If
		Return (L.Get(0))
	End If
	Return CreateMap()
End Sub

'Get the current Library
Public Sub Library As LibraryType
	Return mLibrary
End Sub

'Get elements from the parsed map, always returns a List
Public Sub GetElementsFromMap (m As Map, key As String) As List
	Dim res As List
	If m.ContainsKey(key) = False Then
		res.Initialize
		Return res
	Else
		Dim value As Object = m.Get(key)
		If value Is List Then Return value
		res.Initialize
		res.Add(value)
		Return res
	End If
End Sub

'Get the loaded XML string
Public Sub GetXML As String
	Return mXML
End Sub